﻿
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>
#include "curl/curl.h"
#include <map>
#include "md5.h"

#pragma comment(lib, "libcurl.lib")

// 初始化 apiName 到 task-config 的映射表
std::map<std::string, std::string> initTaskConfigMap()
{
	std::map<std::string, std::string> maps;
	maps["cn_text"]  = "capkey=ocr.cloud,lang=chinese_cn";
	maps["en_text"]  = "capkey=ocr.cloud.english,lang=english";
	maps["bankcard"] = "capkey=ocr.cloud.bankcard";
	maps["bizcard"]  = "capkey=ocr.cloud.bizcard";
	maps["idcard0"]  = "capkey=ocr.cloud.template,property=idcard,templateIndex=0,templatePageIndex=0";
	maps["idcard1"]  = "capkey=ocr.cloud.template,property=idcard,templateIndex=0,templatePageIndex=1";
	maps["vat"]      = "capkey=ocr.cloud.template,property=vat,templateIndex=0,templatePageIndex=0";
	maps["vlcard"]   = "capkey=ocr.cloud.template,property=vlcard,templateIndex=0,templatePageIndex=0";
	maps["dlcard"]   = "capkey=ocr.cloud.template,property=dlcard,templateIndex=0,templatePageIndex=0";
	maps["bl"]       = "capkey=ocr.cloud.template,property=bl,templateIndex=0,templatePageIndex=0";
	maps["pid"]      = "capkey=ocr.cloud.template,property=pid,templateIndex=0,templatePageIndex=0";
	maps["pcn"]      = "capkey=ocr.cloud.template,property=pcn,templateIndex=0,templatePageIndex=0";
	return maps;
}

// 初始化 apiName 到 task-config 的映射表
std::map<std::string, std::string> g_taskConfigMap = initTaskConfigMap();

// 返回 apiName 对应的 taskConfig
std::string getTaskConfig(const char *apiName)
{
	std::map<std::string, std::string>::iterator found = g_taskConfigMap.find(apiName);
	return found == g_taskConfigMap.end() ? "" : found->second;
}

// 获取当前时间，结果为 2017-01-23 08:00:00 这种格式
std::string dateNow()
{
	char sz[0x100];
	time_t now = time(NULL);
	strftime(sz, 0x100, "%Y-%m-%d %H:%M:%S", localtime(&now));
	return sz;
}

std::string md5String(std::string str)
{
	MD5 md5;
	md5.update((unsigned char *)str.c_str(), str.size());
	md5.finalize();
	char *digest = md5.hex_digest();
	std::string sz = (char *)digest;
	delete[] digest;
	return sz;
}

// 生成通用 HTTP HEADER 映射表
std::map<std::string, std::string> getCommonHeaders(const char *appKey, const char *devKey)
{
	std::map<std::string, std::string> headers;
	std::string nowString = dateNow();
	headers["x-request-date"] = nowString;
	headers["x-app-key"] = appKey;
	headers["x-sdk-version"] = "5.1";
	headers["x-session-key"] = md5String(nowString + devKey);
	return headers;
}

// 检查是否为合法的接口名称
bool checkApiName(const char *apiName)
{
	return g_taskConfigMap.find(apiName) != g_taskConfigMap.end();
}

// HTTP POST
FILE *httpPost(const char *url, std::map<std::string, std::string> &headers, const char *filePath)
{
	CURL *handle = curl_easy_init();

	// 设置要访问的 URL
	curl_easy_setopt(handle, CURLOPT_URL, url);

	// 设置 HTTP HEADER
	curl_slist *header_list = NULL;
	std::map<std::string, std::string>::iterator b, e;
	b = headers.begin(), e = headers.end();
	if (b != e)
	{
		for (; b!=e; ++b)
		{
			std::string header = b->first + ": " + b->second;
			header_list = curl_slist_append(header_list, header.c_str());
		}
		curl_easy_setopt(handle, CURLOPT_HTTPHEADER, header_list);
	}

	// POST 方法
	curl_easy_setopt(handle, CURLOPT_POST, true);

	// 读取要发送的文件，设置 POST DATA
	void *data = NULL;
	size_t dataLen = 0;
	if (filePath != NULL)
	{
		FILE *fp = fopen(filePath, "rb");
		if (fp != NULL)
		{
			fseek(fp, 0, SEEK_END);
			dataLen = ftell(fp);
			data = malloc(dataLen);
			fseek(fp, 0, SEEK_SET);
			fread(data, 1, dataLen, fp);
			fclose(fp);
			curl_easy_setopt(handle, CURLOPT_POSTFIELDS, data);
			curl_easy_setopt(handle, CURLOPT_POSTFIELDSIZE, dataLen);
		};
	}
	// 设定为不验证证书和HOST  
    curl_easy_setopt(handle, CURLOPT_SSL_VERIFYPEER, false);
    curl_easy_setopt(handle, CURLOPT_SSL_VERIFYHOST, false);

	// 打开临时文件，返回的结果会写入该文件
	FILE *tmp = tmpfile();
	curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, &fwrite);
	curl_easy_setopt(handle, CURLOPT_WRITEDATA, tmp);
	// 执行 http post
	CURLcode code = curl_easy_perform(handle);

	// 释放资源
	if (data != NULL)
		free(data);
	if (header_list != NULL)
		curl_slist_free_all(header_list);

	// 返回结果
	if (code == CURLE_OK)
		return tmp;
	fclose(tmp);
	return NULL;
}

int main (int argc, char **argv)
{
	const char *apiName = argv[1];  // 接口类型
	const char *filePath = argv[2]; // 图片路径
	const char *apiUrl = "http://ocr.aicloud.com:8541/ocr"; // 正确的地址请从网站上“API说明”处获取
	const char *appKey = "请输入网站上获取的app key";
	const char *devKey = "请输入网站上获取的dev key";

	// 检查参数
	if (argc < 3 || !checkApiName(apiName))
	{
		fprintf(stderr, "使用方法: %s apiName filePath\r\n", argv[0]);
		fprintf(stderr, "参数:\r\n");
		fprintf(stderr, "    apiName  - 调用的接口类型，可取值如下:\r\n\r\n");
		fprintf(stderr, "               cn_text   中文文本识别\r\n");
		fprintf(stderr, "               en_text   英文文本识别\r\n");
		fprintf(stderr, "               idcard0   身份证个人信息页识别\r\n");
		fprintf(stderr, "               idcard1   身份证签发机关页识别\r\n");
		fprintf(stderr, "               vat       票据识别\r\n");
		fprintf(stderr, "               bizcard   名片识别\r\n");
		fprintf(stderr, "               vlcard    行驶证识别\r\n");
		fprintf(stderr, "               dlcard    驾驶证识别\r\n");
		fprintf(stderr, "               bankcard  银行卡识别\r\n");
		fprintf(stderr, "               bl        营业执照识别\r\n");
		fprintf(stderr, "               pid       旅行证件识别\r\n");
		fprintf(stderr, "               pcn       中国护照识别\r\n");
		fprintf(stderr, "\r\n");
		fprintf(stderr, "    filePath - 待识别图片的文件路径\r\n\r\n");
		return 0;
	}

	// 获取 common HTTP HEADER
	std::map<std::string, std::string> headers = getCommonHeaders(appKey, devKey);

	// 在 header 中加入 task-config
	headers["x-task-config"] = getTaskConfig(apiName);

	// 使用 HTTP POST 调用 api
	FILE *response = httpPost(apiUrl, headers, filePath);

	if (response != NULL)
	{
		// 简单的将结果文件内的数据输出到标准输出
		char data[0x1000];
		rewind(response);
		while (!feof(response))
		{
			size_t size = fread(data, 1, 0x1000, response);
			fwrite(data, 1, size, stdout);
		}
		fclose(response);
	}
	return 0;
}
